close all

% Axoneme Program v2.
% Note that the head-tail order has reversed:
% Now the first click should be near the blue O
% and the second click should be near the red *.
axVer=2;

% --------------------- Basic Parameters --------------------- %
 % Name of the image to analyze with GVFC
  iName = 'Example.tif';


dName = iName;  % Name of the image used for display
% Name of output file
wkspName = 'Example.mat'

i0 = 1; % If we are starting fresh, this takes a value of 1
        % otherwise, it takes the value of the index of the
        % last good track.
      
numFr=1000;  % Number of frames to analyze
if(i0==1)
    t0=1;   % First frame to analyze
end
% --------------------- Snake Parameters --------------------- %
if(i0==1)
    % Set these if starting fresh, otherwise if you want to
    % overwrite these values, remove the if-end condition.
    L=35;       % Length of uncompressed filament 
    deltaL=5;   % Characteristic compression length 
    R=2.4;      % Characteristic bending radius
    N=30;       % Number of points
    F=0.1;      % Characteristic compressional force;
   
end
% ---------------------- End Parameters ---------------------- %

dcPix=1.0;  % Distance threshold for double click check

if(i0==1)
    % Initialize the saved position arrays to empty matrices
    Sx=[]; Sy=[];
end

% characteristic energy
E=3*F;

% segment length
l0=L/N;
% spring const
alpha=N*F/(deltaL);
beta=E*R/pi;

% Get the image size
I=double(imread(iName,1));
sz=size(I);

figure(1);
escFlag=0;
i=i0;
while(i<=numFr && escFlag==0)
    goBack=0; % Clear the goback flag
    t=t0+i-1; % Time point

    I=double(imread(iName,t));
    I2=double(imread(dName,t));
    [u,v]=gvfc(I,0.1,20);
    % get average of the top 10 forces
    q=sqrt(u.*u+v.*v);
    q=sort(q(:));
    fTop=mean(q(end-10:end));

    % Place a cap of the speed at which things relax
    gamma=max([1/fTop, 2*alpha]);
    iter=ceil(500*gamma);

    % Display image
    hold off
    imagesc(I2); colormap gray;
    hold all
    quiver(u,v);

    if(i==1)
        % Get the user to click on the points roughly
        % corresponding to the initial contour
        h=impoly(gca,'Closed',false);
        P=h.getPosition;
        delete(h); % clear the polygonal display of points
        
        % Now, let's use the user selected points and interpolate
        % over the N snakepoints to come up with an initial condition.
        dP=diff(P);
        dS=sqrt(dP(:,1).^2+dP(:,2).^2);
        s=[0;cumsum(dS)]; % arclength of user selected points
        s2=linspace(0,s(end),N)';
        x=interp1(s,P(:,1),s2,'pchip');
        y=interp1(s,P(:,2),s2,'pchip');        
        plot(P(:,1),P(:,2),'k'); % display as a blackline.
    else        
        % Get the previous positions
        % (in case there is an error in the tracking and we are
        % backing up to before the error occurred)
        x=Sx(:,i-1); y=Sy(:,i-1);
        
        % Plot the endpoints.
        % March 16, 2016 - Ordering of endpoints flipped.
        plot(x,y,'r',x(1),y(1),'bo',x(end),y(end),'r*');
        
        clickNum=0;        
        while(clickNum<2 && escFlag==0 && goBack==0)
            [xIn,yIn,button]=ginput(1);
            if(~isempty(xIn)) % Ignore the weirdness that pressing enter causes
                switch button
                    case 1
                        % Left mouse click
                        clickNum=clickNum+1;
                        if(clickNum==1)
                            % First click
                            x(1)=xIn; y(1)=yIn;
                            h1=plot(xIn,yIn,'yo'); % draw yellow circle
                        else
                            % The second click
                            x(end)=xIn; y(end)=yIn;
                            h2=plot(xIn,yIn,'y*'); % draw yellow astrisk
                            % Do a double click check
                            endptDist=sqrt((x(end)-x(1))^2+(y(end)-y(1))^2);
                            if(endptDist<dcPix)
                                choice=questdlg('Double click detected.  Was this intended?', ...
                                                'Double click detection', 'Yes', 'No', 'Yes');
                                % Check to see if the close placement was
                                % intentional.  If not go back to the start
                                if(strcmp(choice,'No'))
                                    % Clear the click counter
                                    clickNum=0;
                                    % Clear the points from the plot
                                    delete(h1); delete(h2);
                                end
                            end
                        end
                    case 3
                        % Right mouse click
                        if(clickNum==1)
                            % Clear the first click
                            delete(h1);
                            clickNum=0;
                        end
                    case 28
                        % Left Arrow Pressed
                        if(clickNum==0 && i>2)
                            % Only deal with back arrow if nothing has
                            % been clicked
                            goBack=1;
                            i=i-1
                        end
                    case 27
                        % Escape Key Pressed
                        escFlag=1;
                end
            end % Not empty click check
        end % Click number loop
    end % i==1? Are we getting initial conditions or just endpoints?
    
    if(escFlag==0 && goBack==0)
        [x,y]=snakedeform3b(x,y,alpha,beta,gamma,l0,u,v,iter);
        plot(x,y,'g');
        pause(0.5);
    
        % Store the positions
        Sx(:,i)=x; Sy(:,i)=y;
        
        if(mod(i,10)==0)
            % Autosave the workspace every 10 images
            disp('Saving the workspace.');
            save(wkspName, 'Sx', 'Sy', 'sz', ...
                 't0', 'L', 'deltaL', 'R', 'N', 'F', 'axVer');
        end
        
        i=i+1 % increment the loop counter.
    end
end

i=i-1; % Restore the i value

disp('Saving the workspace.');
save(wkspName, 'Sx', 'Sy', 'sz', ...
    't0', 'L', 'deltaL', 'R', 'N', 'F', 'axVer');