function [x,y] = snakedeform3b(x,y,alpha,beta,gamma,l0,fx,fy,ITER)
% SNAKEDEFORM3  Deform snake in the given external force field
%     [x,y] = snakedeform3(x,y,alpha,beta,gamma,kappa,fx,fy,ITER)
%
%     alpha:   elasticity parameter
%     beta:    rigidity parameter
%     gamma:   viscosity parameter
%     fx,fy:   external force field

%      Chenyang Xu and Jerry L. Prince, 4/1/95, 6/17/97
%      Copyright (c) 1995-97 by Chenyang Xu and Jerry L. Prince
%      Image Analysis and Communications Lab, Johns Hopkins University

% Modified on 2/17/2016 by A. Bae for open boundary snakes
% and for nonzero spring length.
% Modified on 2/18/2016 for fixed endpoints.

% generates the parameters for snake

N = length(x);

% rename the spring constant to k
k = alpha;

% produce the five diagonal matrix
B = zeros(1,N);
B(1:3) = [6,-4,1];
% Generate the 5 diagonal 4th derivative matrix
B = toeplitz(B,B);

% Correct for boundary conditions
betaOpenBD = [ 2,-4, 2, 0; ...
              -2, 5,-4, 1];
B(1:2,1:4) = betaOpenBD;
B(end:-1:end-1,end:-1:end-3) = betaOpenBD;

% Give it the appropriate weight
B=-beta*B;


for count = 1:ITER
    % Construct the tridiagonal elasticity matrix
    dx=diff(x);
    dy=diff(y);
    r=sqrt(dx.*dx+dy.*dy);
    alpha=k*(1-l0./r);
    A=zeros(N);
    A(1:N-1,1:N-1)=A(1:N-1,1:N-1)-diag(alpha);
    A(2:N,2:N)=A(2:N,2:N)-diag(alpha);
    A=A+diag(alpha,1)+diag(alpha,-1);
    
    % Invert the matrix
    invAI = inv( -A -B + gamma * diag(ones(1,N)));

    vfx = interp2(fx,x,y,'*linear');
    vfy = interp2(fy,x,y,'*linear');

    x0 = x(1); xN = x(N);
    y0 = y(1); yN = y(N);
    
    % deform snake
    x = invAI * (gamma* x + vfx);
    y = invAI * (gamma* y + vfy);
    
  
    
    x(1) = x0; x(N) = xN;
    y(1) = y0; y(N) = yN;
end
